# Sage version information for shell scripts
# This file is auto-generated by the sage-update-version script, do not edit!
SAGE_VERSION='9.4'
SAGE_RELEASE_DATE='2021-08-22'
SAGE_VERSION_BANNER='SageMath version 9.4, Release Date: 2021-08-22'
